// PCSCApp.cpp : Defines the entry point for the console application.
//

#include <tchar.h>
#include <Winscard.h>
#include <cassert>
#include <iostream>
#include <fstream>

// Forward declarations of functions included in this code module:
using namespace std;

SCARD_READERSTATE rgscState[MAXIMUM_SMARTCARD_READERS];

const TCHAR NFC_READER_NAME[] = _T("Macally NFC PCSC Reader");

int _tmain(int argc, _TCHAR* argv[])
{
	SCARDCONTEXT card_context;
	
	if(SCARD_S_SUCCESS != SCardEstablishContext(SCARD_SCOPE_USER, NULL, NULL, &card_context))
	{
		wcerr << _T("Smart card service is not running!") << endl;
		return 3;
	}

	// Determine which readers are available.
	DWORD             cchReaders = SCARD_AUTOALLOCATE;
	LPTSTR            szReaders, szRdr;
	LONG lReturn = SCardListReaders(card_context,
		NULL,
		(LPTSTR)&szReaders,
		&cchReaders );
	if ( SCARD_S_SUCCESS != lReturn )
	{
		wcerr << _T("Failed SCardListReaders") << endl;
		return 4;
	}
	// Place the readers into the state array.

	szRdr = szReaders;
	DWORD             dwI, find_count;
	for (dwI = 0, find_count = 0; dwI < MAXIMUM_SMARTCARD_READERS; dwI++ )
	{
		if ( 0 == *szRdr )
			break;
		if (_tcsncmp(szRdr, NFC_READER_NAME, sizeof(NFC_READER_NAME)/sizeof(TCHAR)-1) == 0)
		{
			rgscState[find_count].szReader = szRdr;
			rgscState[find_count].dwCurrentState = SCARD_STATE_UNAWARE;
			++find_count;
		}
		szRdr += lstrlen(szRdr) + 1;
	}
	if (find_count == 0)
	{
		wcerr << _T("Error: Can not find any Macally PCSC NFCReader!") << endl;
		return 7;
	}

	lReturn = SCardGetStatusChange(card_context,
		INFINITE,
		rgscState,
		find_count );
	if ( SCARD_S_SUCCESS != lReturn )
	{
		wcerr << _T("Failed SCardGetStatusChange!") << endl;
		return 5;
	}

	SCARDHANDLE card = 0;
	DWORD protocol = 0;
	if (rgscState[0].dwEventState & SCARD_STATE_PRESENT) 
	{
		lReturn = SCardConnect(card_context,
			rgscState[0].szReader,
			SCARD_SHARE_EXCLUSIVE,
			SCARD_PROTOCOL_T1,
			&card,
			&protocol
			);
		if ( SCARD_S_SUCCESS != lReturn )
		{
			wcerr << _T("Failed SCardConnect()!") << endl;
			return 6;
		}
		
		//Seleect Master File

		BYTE apdu_cmd[] = { 0xA0, 0xA4, 0x00, 0x00, 0x02, 0x3F, 0x00 };

		BYTE recv[600];
		DWORD recv_len = sizeof(recv);

		{
			memset(recv, 0, sizeof(recv));
			recv_len = sizeof(recv);

			printf("Transmit:");
			for (DWORD i = 0; i < sizeof(apdu_cmd); ++i)
			{
				printf(" %02X", apdu_cmd[i]);
			}
			printf("\n");

			lReturn = SCardTransmit(
				card, 
				SCARD_PCI_T1,
				apdu_cmd,
				sizeof(apdu_cmd),
				NULL,
				recv,
				&recv_len
				);

			if ( SCARD_S_SUCCESS != lReturn )
			{
				wcerr << _T("Command failed!!") << endl;
			} else {
				printf("Response:");
				for (DWORD i = 0; i < recv_len; ++i)
				{
					printf(" %02X", recv[i]);
				}
				printf("\n");
			}
		}

		SCardDisconnect(card, SCARD_LEAVE_CARD);
	} else {
		wcerr << "There is no card on the reader." << endl;
	}

	SCardFreeMemory( card_context, szReaders );

	SCardReleaseContext(card_context);

	return 0;
}

